<?php


/**
* Jmol resource type, written by Dan Stowell 2006
* Open-source under the Gnu Public License (GPL)
*
* Some notes:
* 
* I'm concerned to try and make sure this resource type doesn't make 
* it easy to disadvantage people on the basis of computer power or 
* (of course) of disability. So a couple of design choices:
*
*  - Don't allow the user to select massive viewer sizes.
*    A size of 800px might look nice but will be a big processing impact
*    on less powerful computers, and will reduce usability on smaller screens.
*  - Don't provide an option to turn spinning on by default. (Spinning takes
*    more processing power)
*  - Don't allow the user to hide the download link. If students want to 
*    deal with the data file in some other way they should be able to.
* 
* 
*/




/**
* Extend the base resource class for jmol resources
*
*/
class resource_jmol extends resource_base {

function resource_jmol($cmid=0) {
    parent::resource_base($cmid);
}



/**
* Add new instance of jmol resource
*
* Create alltext field before calling base class function.
*
* @param    resource object
*/
function add_instance($resource) {

    $resource->options = $this->options_to_string($resource);

    return parent::add_instance($resource);
}


/**
* Update instance of jmol resource
*
* Create alltext field before calling base class function.
*
* @param    resource object
*/
function update_instance($resource) {
    // "alltext" field will contain the arbitrary Jmol initialisation commands (no max length)
    // "options" field will contain the standard options (max length 255 chars)
    $resource->options = $this->options_to_string($resource);

    return parent::update_instance($resource);
}


/**
* Collapse the Jmol-specific form options into a database-safe string
*/
function options_to_string($resource){
    return 'size='.intval($resource->size)
                 .'&atoms='.((float)$resource->atoms)
                 .'&cont_atoms='.intval($resource->cont_atoms)
                 .'&cont_spin='.intval($resource->cont_spin)
                 .'&cont_cnsl='.intval($resource->cont_cnsl)
//                 .'&cont_struct='.intval($resource->cont_struct)
                 .'&struct='.preg_replace('/\W+/is', '', $resource->struct)
                 ;
}

/**
* Expand the Jmol-specific form options and add them on to the object passed in
*/
function options_from_string(&$resource){
    $opts = explode('&', $resource->options);
    foreach($opts as $opt){
        if(trim($opt) && ($parts = explode('=', $opt, 2)) && sizeof($parts)==2){
            $resource->$parts[0] = $parts[1];
        }
    }
}


/**
* Display the jmol resource
*
* Displays a jmol resource embedded, in a frame, or in a popup.
* Output depends on type of jmol resource.
*
* @param    CFG     global object
*/
function display() {
    global $CFG, $THEME, $jmol_applet_has_been_initialised;

    // The global variable "$filter_jmol_has_initialized"
    //  is used by BOTH the Jmol filter and the Jmol resource type
    //  to ensure that the Jmol applet code is only ever once 
    //  written to any given web page.

/// Set up generic stuff first, including checking for access
    parent::display();

/// Set up some shorthand variables
    $cm = $this->cm;
    $course = $this->course;
    $resource = $this->resource;


    $pagetitle = strip_tags($course->shortname.': '.format_string($resource->name));


        if ($CFG->slasharguments) {
            $relativeurl = "/file.php/{$course->id}/{$resource->reference}";
            if ($querystring) {
                $relativeurl .= '?'.$querystring;
            }
        } else {
            $relativeurl = "/file.php?file=/{$course->id}/{$resource->reference}";
        }
        $fullurl = "$CFG->wwwroot$relativeurl";


    /// Add an entry to the log

    add_to_log($course->id, "resource", "view", "view.php?id={$cm->id}", $resource->id, $cm->id);


        $strdirectlink = get_string("directlink", "resource");
            print_header($pagetitle, $course->fullname, "$this->navigation <a title=\"$strdirectlink\" target=\"$CFG->framename\" href=\"$fullurl\"> ".format_string($resource->name,true)."</a>", "", "", true, update_module_button($cm->id, $course->id, $this->strresource), navmenu($course, $cm, "self"));




    // Jmol requires that we convert our full URL to a relative URL.
    // Otherwise it displays a warning message and refuses to run!
    
    $u = $CFG->wwwroot;

    if(preg_match('|https?://.*?/|', $u)){
      $relurl = preg_replace('|https?://.*?/|', '', $u);
    }else{
      $relurl = ''; // This will typically be the case if Moodle is the web root
    }
    $numdirs = substr_count($_SERVER['PHP_SELF'], '/') - 1;
    if($numdirs==0){
      $relurl = './' . $relurl;
    }else{
      $relurl = str_repeat('../', $numdirs) . $relurl;
    }

    
    
    // Grab the Jmol-specific settings from the option string
    $this->options_from_string($resource);

    //echo "<pre>About to display() with this data:\n".print_r($resource, true)."</pre>";



        if (trim($resource->summary)) {
            $formatoptions->noclean = true;
            print_simple_box(format_text(trim($resource->summary), FORMAT_MOODLE, $formatoptions, $course->id), "center");
        }



// The Jmol javascript code

$size = $resource->size;
$initcode = $resource->alltext;

if(!$jmol_applet_has_been_initialised){
  $jmol_applet_has_been_initialised = true;
  ?>
<script type="text/javascript" src="<?php echo $relurl ?>/mod/resource/type/jmol/Jmol.js"></script><noscript>
The 3D viewer uses Javascript to create the Java viewing window. If you are unable 
to make use of this technology, you can use the "download" link to download the 
file. If you are still unable to make use of the information, contact your 
<?php echo get_string('teacher') ?>.
</noscript>
<script type="text/javascript">
   jmolInitialize("<?php echo $relurl ?>/mod/resource/type/jmol/");
</script>
<?php
}
?>

<form style="text-align: center; ">
  <script type="text/javascript">
   jmolApplet(<?php echo $size ?>, "load ../..<?php echo $relativeurl;
     echo '; wireframe off; ' . $resource->struct . ' on';
     echo '; spacefill ';
     switch($resource->atoms){
       case 0.0:
         echo 'off';
         break;
       case 1.0:
         echo '100%';
         break;
       default:
         echo '20%';
     }
     if($initcode){
       echo ";".str_replace('"', "'", str_replace("\n", '; ', trim($initcode)));
     }
     ?>");
jmolBr();
<?php  
if($resource->cont_atoms){
  ?>
  jmolHtml("Atoms ");
  jmolRadioGroup([
   ["spacefill off", "off"<?php if($resource->atoms==0.0) echo ', "checked"' ?>],
   ["spacefill 20%", "20%"<?php if($resource->atoms==0.2) echo ', "checked"' ?>],
   ["spacefill 100%", "100%"<?php if($resource->atoms==1.0) echo ', "checked"' ?>]
  ]);
  <?php
}
/*
if($resource->cont_struct){
  ?>
  jmolMenu([
   ["wireframe on", "Wireframe"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["backbone on", "Backbone"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["ribbons on", "Ribbons"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["trace on", "Trace"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["cartoons on", "Cartoons"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["strands on", "Strands"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["meshribbon on", "Meshribbon"<?php if($resource->atoms==0.0) echo ', true' ?>],
   ["rockets on", "Rockets"<?php if($resource->atoms==0.0) echo ', true' ?>]
  ]);
  <?php
}
*/
if($resource->cont_spin){
  ?>
  jmolHtml("&nbsp; &nbsp;");
  jmolCheckbox("spin on", "spin off", "Spin");
  <?php
}
if($resource->cont_cnsl){
  ?>
  jmolHtml("&nbsp; &nbsp;");
  jmolButton('console', '<?php echo get_string('launchconsole', 'resource_jmol') ?>', 'jmol_cont_cnsl');
  <?php
}
?>

</script>
<a href="../..<?php echo $relativeurl ?>" title="Download the structure data file"><img align="absmiddle" height="17" width="17" src="type/jmol/download.gif" /></a>&nbsp;&nbsp;
<?php
helpbutton('jmol', 'Help with Jmol', 'jmol');
?>
</form>
<?php


        print_spacer(20,20);
        print_footer($course);


}



/**
* Setup a new jmol resource
*
* Display a form to create a new or edit an existing jmol resource
*
* @param    form                    object
* @param    CFG                     global object
* @param    usehtmleditor           global integer
* @param    RESOURCE_WINDOW_OPTIONS global array
*/
function setup($form) {
    global $CFG, $editorfields, $usehtmleditor, $RESOURCE_WINDOW_OPTIONS;

    $editorfields = 'summary'; // Prevents the HTMLeditor coming in for the script box

    parent::setup($form);

    $strfilename = get_string("location");
    $strnote     = get_string("note", "resource");
    $strchooseafile = get_string("chooseafile", "resource");

    foreach ($RESOURCE_WINDOW_OPTIONS as $optionname) {
        $stringname = "str$optionname";
        $$stringname = get_string("new$optionname", "resource");
        $window->$optionname = "";
        $jsoption[] = "\"$optionname\"";
    }

    $frameoption = "\"framepage\"";
    $popupoptions = implode(",", $jsoption);
    $jsoption[] = $frameoption;
    $alloptions = implode(",", $jsoption);



    if ($form->instance) {     // Re-editing
        if (!$form->popup) {
            $windowtype = "page";   // No popup text => in page
            foreach ($RESOURCE_WINDOW_OPTIONS as $optionname) {
                $defaultvalue = "resource_popup$optionname";
                $window->$optionname = $CFG->$defaultvalue;
            }
        } else {
            $windowtype = "popup";
            $rawoptions = explode(',', $form->popup);
            foreach ($rawoptions as $rawoption) {
                $option = explode('=', trim($rawoption));
                $optionname = $option[0];
                $optionvalue = $option[1];
                if ($optionname == 'height' or $optionname == 'width') {
                    $window->$optionname = $optionvalue;
                } else if ($optionvalue) {
                    $window->$optionname = 'checked="checked"';
                }
            }
        }
    } else {
        foreach ($RESOURCE_WINDOW_OPTIONS as $optionname) {
            $defaultvalue = "resource_popup$optionname";

            if ($optionname == 'height' or $optionname == 'width') {
                $window->$optionname = $CFG->$defaultvalue;
            } else if ($CFG->$defaultvalue) {
                $window->$optionname = 'checked="checked"';
            }
        }

        $windowtype = ($CFG->resource_popup) ? 'popup' : 'page';
        if (empty($form->options)) {
            $form->options = 'frame';
            $form->reference = $CFG->resource_defaulturl;
        }
    }
    if (empty($form->reference)) {
        $form->reference = $CFG->resource_defaulturl;
    }

    // Expand the Jmol-specific options from the single string
    $this->options_from_string($form);

    // Fill in default values if not set (should only be needed when creating new)
    if(!isset($form->size)) $form->size = 300;
    if(!isset($form->atoms)) $form->atoms = 0.2;
    if(!isset($form->struct)) $form->struct = 'wireframe';
    if(!isset($form->cont_atoms)) $form->cont_atoms = 1;
    if(!isset($form->cont_spin)) $form->cont_spin = 1;
    if(!isset($form->cont_cnsl)) $form->cont_cnsl = 0;


    include("$CFG->dirroot/mod/resource/type/jmol/jmol.html");

    parent::setup_end();
}


}

?>
